#ifndef __COMMS_H__
#define __COMMS_H__

// PIN_B7 = transmit
// PIN_B5 = receive
#use rs232(UART1, baud=9600, receive_buffer=16, timeout=250, errors, stream=STREAM_UART1)

// these are commands that can be sent by the master.
// if the secondary is responding to these, it's ack will be a message
// with bit7 of the command set (a command sent with 0x00 will be responded
// with a 0x80).
typedef enum
{
   SET_LEDS = 0,   //payload[0] is a bitmap of which LEDs to turn on,
                  //payload[1] is a bitmap of which LEDs to turn off.
                  //bit0=red, bit1=yellow, bit=green
                  
   WRITE_EE = 1,  //payload[0] is address, payload[1] is value
   
   READ_EE = 2,   //payload[0] is address, 
                  //in response packet payload[0] is value.
                  
   READ_BUTTONS = 3, //in response packet payload[0] is a bitmap of buttons pressed
   
   GET_ADC = 4,  //in response packet payload[0] is ADC
   
   UNKNOWN_COMMAND = -1
} command_t;

//see SET_LEDS above
typedef enum
{
   LED_BITMAP_RED = 1,
   LED_BITMAP_YELLOW = 2, 
   LED_BITMAP_GREEN = 4
} leds_bitmap_t;

// all transmissions and responses are exactly 3 bytes long.
typedef struct
{
   command_t cmd;
   unsigned int8 payload[2];
} packet_t;

// receive a packet.  returns TRUE if success, and packet saved to 'p'
int1 CommsRxPacket(packet_t *p)
{
   p->cmd = fgetc(STREAM_UART1);
   if (RS232_ERRORS == 0) return FALSE;   //timeout
   
   p->payload[0] = fgetc(STREAM_UART1);
   if (RS232_ERRORS == 0) return FALSE;   //timeout
   
   p->payload[1] = fgetc(STREAM_UART1);
   if (RS232_ERRORS == 0) return FALSE;   //timeout

   return TRUE;
}

// transmit a packet
void CommsTxPacket(packet_t *p)
{
   fputc(p->cmd, STREAM_UART1);
   fputc(p->payload[0], STREAM_UART1);
   fputc(p->payload[1], STREAM_UART1);
}

// sends packet 'pPacket' to secondary E3 and waits for response.
// response saved back to 'pPacket'.
// returns TRUE if success
int1 CommsTransaction(packet_t *pPacket)
{
   packet_t pTx;
   
   memcpy(&pTx, pPacket, sizeof(pTx));
   
   CommsTxPacket(&pTx);
   
   if (!CommsRxPacket(pPacket))
      return(FALSE);
   
   return ((pTx.cmd | 0x80) == pPacket->cmd);
}

// use SET_LEDS command to set LEDs on secondary E3.
// return TRUE if success
int CommsSetLEDs(unsigned int8 onBitmap, unsigned int8 offBitmap)
{
   packet_t p;
   
   p.cmd = SET_LEDS;
   p.payload[0] = onBitmap;
   p.payload[1] = offBitmap;
   
   return(CommsTransaction(&p));
}

// set red led on.  returns TRUE if success.
int1 CommsSetRedLEDOn(void)
{
   return(CommsSetLEDs(LED_BITMAP_RED, 0));
}

// set yellow led on.  returns TRUE if success.
int1 CommsSetYellowLEDOn(void)
{
   return(CommsSetLEDs(LED_BITMAP_YELLOW, 0));
}

// set green led on.  returns TRUE if success.
int1 CommsSetGreenLEDOn(void)
{
   return(CommsSetLEDs(LED_BITMAP_GREEN, 0));
}

// set red led off.  returns TRUE if success.
int1 CommsSetRedLEDOff(void)
{
   return(CommsSetLEDs(0, LED_BITMAP_RED));
}

// set yellow led off.  returns TRUE if success.
int1 CommsSetYellowLEDOff(void)
{
   return(CommsSetLEDs(0, LED_BITMAP_YELLOW));
}

// set green led off.  returns TRUE if success.
int1 CommsSetGreenLEDOff(void)
{
   return(CommsSetLEDs(0, LED_BITMAP_GREEN));
}

// write 'value' to 'location' on secondary E3.  return TRUE if success.
int1 CommsWriteLocation(int location, int value)
{
   packet_t p;
   
   p.cmd = WRITE_EE;
   p.payload[0] = location;
   p.payload[1] = value;
   
   return(CommsTransaction(&p));
}

// read value from 'location' on secondary E3.  saves to 'value'.
// returns TRUE if ok.
int1 CommsReadLocation(int location, int *value)
{
   packet_t p;
   
   p.cmd = READ_EE;
   p.payload[0] = location;
   
   if (!CommsTransaction(&p))
      return(FALSE);
   
   *value = p.payload[0];
   
   return(TRUE);
}

// read buttons on secondary E3.  saves to 'value'.
// returns TRUE if ok.
int1 CommsReadButtons(int *value)
{
   packet_t p;
   
   p.cmd = READ_BUTTONS;
   
   if (!CommsTransaction(&p))
      return(FALSE);
   
   *value = p.payload[0];
   
   return(TRUE);
}

int1 CommsReadADC(int *value)
{
   packet_t p;
   
   p.cmd = GET_ADC;
   p.payload[0] = 0;
   p.payload[1] = 0;
   
   if (!CommsTransaction(&p))
      return(FALSE);
   
   *value = p.payload[0];
   
   return(TRUE);
}

#endif
